<?php

namespace Drupal\semantic_search\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Url;
use Drupal\Core\Form\FormStateInterface;
use Drupal\semantic_search\Services\EmbeddingStorageService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\semantic_search\Http\PineconeClient;
use Drupal\Core\Database\Connection;

/**
 * Provides a confirm form to delete all Pinecone items.
 */
class DeleteConfirmForm extends ConfirmFormBase {

  /**
   * The database service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The Pinecone client.
   *
   * @var \Drupal\semantic_search\Http\PineconeClient
   */
  protected $client;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->database = $container->get('database');
    $instance->client = $container->get('semantic_search.pinecone_client');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'semantic_search_embeddings_delete_confirm';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Are you sure you want to delete all items in your %storage index?', ['%storage' => \Drupal::config('semantic_search.settings')->get('embedding_storage')]);
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->t('This will delete all items in your %storage index.', ['%storage' => \Drupal::config('semantic_search.settings')->get('embedding_storage')]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return Url::fromRoute('semantic_search.settings_form');
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    switch (\Drupal::config('semantic_search.settings')->get('embedding_storage')) {
      case EmbeddingStorageService::LOCAL_STORAGE_KEY:
        $embedding_service  = \Drupal::service('semantic_search.embedding_storage');
        $embedding_service->EraseLocalFile();
        break;
      case EmbeddingStorageService::PINECONE_STORAGE_KEY:
        $results = $this->database->query('SELECT entity_type FROM {semantic_search_embeddings}');

        foreach ($results as $result) {
          $this->client->delete([], TRUE, EmbeddingStorageService::PINECONE_NAMESPACE);
        }

        $this->messenger()->addStatus($this->t('All items have been deleted in Pinecone.'));
        $form_state->setRedirect('semantic_search.settings_form');
        break;
        // @todo Postgres.
      case EmbeddingStorageService::POSTGRES_STORAGE_KEY:
        break;
    }

  }

}
