<?php

namespace Drupal\semantic_search\Services;

/**
 * Service to calculate embeddings.
 *
 * @package Drupal\semantic_search\Services
 */
class EmbeddingVectorizationService {

  /**
   * The storage options and vector sizes.
   */
  const storage_options = [
    'openai' => [
      'key' => 'openai',
      'label' => 'Open AI',
      'vector_size' => 1536,
    ],
    'custom' => [
      'key' => 'custom',
      'label' => 'Custom',
      'vector_size' => 768,
    ],
  ];

  /**
   * The OpenAI storage key.
   */
  const OPENAI_KEY = 'openai';

  /**
   * The custom API key.
   */
  const CUSTOM_API_KEY = 'custom';

  /**
   * Constructs for EmbeddingVectorizationService.
   */
  public function __construct() {
  }

  /**
   * Get embedding.
   *
   * @return array
   *   OpenAI embedding.
   */
  public function getEmbedding($source) {
    switch (EmbeddingVectorizationService::getVectorizationService()) {
      case EmbeddingVectorizationService::OPENAI_KEY:
        $openai_key = \Drupal::config('semantic_search.openai_client')->get('openai_key');
        $client = \OpenAI::client($openai_key);
        $response = $client->embeddings()->create([
          'model' => 'text-embedding-ada-002',
          'input' => strip_tags($source),
        ]);

        foreach ($response->embeddings as $embedding) {
          $embedding = $embedding->embedding;
        }
        break;
      case EmbeddingVectorizationService::CUSTOM_API_KEY:

        // Generated by curl-to-PHP: http://incarnate.github.io/curl-to-php/
        $ch = curl_init();

        $custom_vectorization_api_config = \Drupal::config('semantic_search.custom_vectorization_api');

        $search_string = str_replace('"', "", $source);
        $search_string = str_replace(array("\r", "\n"), '', $search_string);

        curl_setopt($ch, CURLOPT_URL, $custom_vectorization_api_config->get('url'));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, "{ \"input\": \"" . $search_string . "\" }");

        $headers = array();
        $headers[] = 'Content-Type: application/json';
        $headers[] = 'Authorization: basic ' . $custom_vectorization_api_config->get('auth_token');
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        $embedding = json_decode($result)->embedding;
        if (curl_errno($ch)) {
          echo 'Error:' . curl_error($ch);
        }
        curl_close($ch);
        break;
    }


    return $embedding;
  }

  public static function getVectorizationService() {
    $vectorization_service = \Drupal::config('semantic_search.settings')->get('embedding_vectorization');
    return $vectorization_service;
  }

  public static function getVectorSize() {
    $vectorization_service = EmbeddingVectorizationService::getVectorizationService();
    return EmbeddingVectorizationService::storage_options[$vectorization_service]['vector_size'];
  }

}
